from whacc import utils
from whacc.utils import info
import h5py
from whacc import model_maker, image_tools
import numpy as np


def make_example_h5():
    h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'
    h5_new = '/Users/phil/Desktop/feature_example2.h5'
    utils.print_h5_keys(h5_in)
    with h5py.File(h5_in, 'r') as h:
        with h5py.File(h5_new, 'w') as h2:
            h2['images'] = h['images'][655102:656252]
            h2['labels'] = h['labels'][655102:656252]
            h2['frame_nums'] = [len(h2['labels'][:])]

    # convert to feature data
    # h5_3lag = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag.h5'
    h5_feature_data = h5_new.replace('.h5', '_feature_data.h5')
    in_gen = image_tools.ImageBatchGenerator(500, h5_new, label_key='labels')
    RESNET_MODEL = model_maker.load_final_model()
    utils.convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)

    # h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
    # generate all the modified features (41*2048)+41 = 84,009
    utils.standard_feature_generation(h5_feature_data)
    return h5_feature_data


def get_example_h5_selected_2105_features(h5_feature_data=None):
    if h5_feature_data is None:
        h5_feature_data = '/Users/phil/Desktop/feature_example2_feature_data.h5'
    d = utils.load_feature_data()
    feature_index = d['final_selected_features_bool']

    with h5py.File(h5_feature_data, 'r') as h:
        all_x = []
        all_y = np.asarray(h['labels'][:])
        for k in d['feature_list_unaltered']:
            x = h[k][:]
            if len(x.shape) == 1:
                x = x[:, None]
            all_x.append(x)
    all_x = np.hstack(all_x)
    all_x = all_x[:, feature_index]
    return all_x, all_y


all_x, all_y = get_example_h5_selected_2105_features()






from whacc import model_maker

from whacc.model_maker import *


import h5py

from tensorflow.keras.layers import Dense, LSTM, Flatten, TimeDistributed, Conv2D, Dropout, GRU

from tensorflow.keras.utils import plot_model
from tensorflow.keras.models import Model
from tensorflow.keras import Sequential
from tensorflow import keras
from tensorflow.keras import applications

import tensorflow as tf
from tqdm.notebook import tqdm
from tqdm import tqdm
import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
from datetime import datetime
import pytz
import json
from math import isclose, sqrt
from IPython import display

"""# define some funcitons"""
def image_transform(raw_X):
  IMG_SIZE = 96
  if len(raw_X.shape) == 4 and raw_X.shape[3] == 3:
      rgb_batch = copy.deepcopy(raw_X)
  else:
      rgb_batch = np.repeat(raw_X[..., np.newaxis], 3, -1)
  rgb_tensor = tf.cast(rgb_batch, tf.float32)  # convert to tf tensor with float32 dtypes
  rgb_tensor = (rgb_tensor / 127.5) - 1  # /127.5 = 0:2, -1 = -1:1 requirement for mobilenetV2
  rgb_tensor = tf.image.resize(rgb_tensor, (IMG_SIZE, IMG_SIZE))  # resizing
  # IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
  return rgb_tensor

def inds_sorted_data(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through
  """
    log_ind = np.where(key_name == a['logs_names'])[0][0]
    val_list = a['all_logs'][:, log_ind]
    if max_or_min == 'max':
        max_arg_sort = np.flip(np.argsort(val_list)) + 1
    elif max_or_min == 'min':
        max_arg_sort = np.argsort(val_list) + 1
    else:
        raise ValueError("""max_or_min must be a string set to 'max' or 'min'""")
    return max_arg_sort


def sorted_loadable_epochs(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through
  """
    arg_sort_inds = inds_sorted_data(data, key_name, max_or_min)
    arg_sort_inds[np.argmax(arg_sort_inds)] = -1
    saved_epoch_numbers = np.asarray(list(data['info']['epoch_dict'].keys()))
    sorted_loadable_epochs_out = []
    for k in arg_sort_inds:
        if k in saved_epoch_numbers:
            sorted_loadable_epochs_out.append(k)
    return sorted_loadable_epochs_out


def get_automated_model_info_TL(BASE_H5, image_source_h5_directory_ending, test_data_dir, data_string_key="data"):#keep
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + data_string_key + a
    base_dir_all_h5s = BASE_H5 + a + data_string_key + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    # pdb.set_trace()
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    # labels_dir = base_data_dir + a + "ALT_LABELS" + a
    # print('\nFOR LABELS,0 is train set, 1 is val set')
    # label_h5_list = utils.get_h5s(labels_dir)
    # print('\nSelect from the following label structures...')
    # print(labels_dir)
    # label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    # h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    # h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()



def re_build_model_TL(model_name_str, class_numbers, base_learning_rate=0.00001,
                      dropout_val=None, IMG_SIZE=96, labels=None, reload_weights_file=None, num_layers_unfreeze=0):#keep
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    # base_model.summary()
    num_layers_in_base_model = len(base_model.layers)
    base_model_layer_names = [k.name for k in base_model.layers]
    base_model.trainable = False  ##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$
    len_base_model = len(base_model.layers)
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]
    if reload_weights_file is not None:
        model.load_weights(reload_weights_file)  # load model weights

    # for i, k in enumerate(model.layers):
    #   if i >= len_base_model-5:
    #     k.trainable = True
    #   else:
    #     k.trainable = False
    relu_layers = []
    for i, k in enumerate(model.layers):
        if 'relu' in k.name.lower():
            relu_layers.append(i)
    relu_layers.append(9999999)
    # relu_layers = np.flip(np.asarray(relu_layers)+1)
    relu_layers = np.flip(np.asarray(relu_layers))

    # num_layers_unfreeze =1          #0 means freeze entire model 1 means base model forzen 2 one more (group)laye runfrozen etc
    for i, k in enumerate(model.layers):
        if i >= relu_layers[
            num_layers_unfreeze] and 'batchnorm' not in k.name.lower():  # LK:J:LKJD:LKJD:LKDJ:LDKJ:DLKJD:LKJD:LKJD:LKJD:LKDJ:LKDJLK:DJ
            k.trainable = True
        else:
            k.trainable = False

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    model.num_layers_in_base_model = num_layers_in_base_model
    model.base_model_layer_names = base_model_layer_names
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model


def add_lstm_to_model(model_in, num_layers_in_base_model, base_learning_rate=10 ** -5, lstm_len=7):#keep
    # model_in.base_model_layer_names
    base_model = Model(model_in.input, model_in.layers[num_layers_in_base_model - 1].output)
    model_out = Sequential()
    model_out.add(TimeDistributed(base_model, input_shape=(lstm_len, 96, 96, 3)))
    model_out.add(TimeDistributed(Flatten()))
    model_out.add(LSTM(256, activation='relu', return_sequences=False))
    model_out.add(Dense(64, activation='relu'))
    # is 64 here the final number of features if so i want both of these to be bigger, check what I did for the OG models
    model_out.add(Dropout(.2))
    model_out.add(Dense(1, activation='sigmoid'))

    optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
    loss = keras.losses.BinaryCrossentropy()
    metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5), keras.metrics.AUC(name='auc')]
    model_out.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    return model_out


def rename_content_folder(instr):# keep
    if isinstance(instr, str) and '/content/' in instr:
        return '/Users/phil/Desktop/content' + instr.split('content')[-1]
    else:
        return instr

# these last 2 are used to change the directories easily when running on local
def foo_rename2(instr):#keep
    if isinstance(instr, str) and '/My Drive/' in instr:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]
    else:
        return instr


def make_D_local(D):#keep
    for key in D:
        if isinstance(D[key], str):
            D[key] = foo_rename2(D[key])
    for key in D['epoch_dict']:
        if isinstance(D['epoch_dict'][key], str):
            D['epoch_dict'][key] = foo_rename2(D['epoch_dict'][key])
    return D

"""# Load all_data for reload models and settings """

def save_obj(obj, name):#keep
    with open(name + '.pkl', 'wb') as f:
        pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)

def load_obj(name):#keep
    with open(name + '.pkl', 'rb') as f:
        return pickle.load(f)

def foo_rename(instr):#keep
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]

all_data = utils.load_obj(foo_rename('/content/gdrive/My Drive/colab_data2/all_data'))

h5_in = '/Users/phil/Desktop/pipeline_test/AH0407x160609_3lag.h5'
key_name = 'acc_test'
max_or_min = 'max'

names = [os.path.basename(data['full_name']) for data in all_data]
lag_names, lag_inds = utils.lister_it(names, keep_strings='3lag__regular', return_bool_index=True)
regular_names, regular_inds = utils.lister_it(names, keep_strings='regular__regular', remove_string='__EfficientNetB7__', return_bool_index=True)

all_inds = np.logical_or(regular_inds, lag_inds)
all_inds = np.where(all_inds)[0]
# all_names = regular_names+lag_names

batch_size = 500
data_in = np.random.rand(100, 96, 96, 3)
final_save_dir = foo_rename('/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/final_CNN_predictions_16_count/')
utils.make_path(final_save_dir)

for model_ind in all_inds:
    data = all_data[model_ind]
    best_epochs = sorted_loadable_epochs(data, key_name, max_or_min)
    reload_model_epoch = best_epochs[0]
    D = copy.deepcopy(data['info'])

    epoch_model_file = D['epoch_dict'][reload_model_epoch]
    epoch_model_file = foo_rename(epoch_model_file)

    model = re_build_model_TL(D['model_name_str'],
                              D['class_numbers'],
                              base_learning_rate=0,
                              dropout_val=0,
                              IMG_SIZE=D['IMG_SIZE'],
                              reload_weights_file=epoch_model_file,
                              num_layers_unfreeze=0)

    i1 = 0
    i2 = 0
    cnn_predictions = []
    for k in tqdm(range(int(np.ceil(data_in.shape[0]/batch_size)))):
        i2 += batch_size ##############################
        x_temp = image_transform(data_in[i1:i2])
        cnn_predictions.append(model.predict(x_temp))
        i1 += batch_size ##############################
    cnn_predictions = np.concatenate(cnn_predictions)
    utils.save_obj(cnn_predictions, final_save_dir+os.path.basename(data['full_name']))



# from google.colab import drive
# drive.mount('/content/gdrive')
# drive._mount('/content/gdrive')
from whacc import model_maker

from whacc.model_maker import *


import h5py

from tensorflow.keras.layers import Dense, LSTM, Flatten, TimeDistributed, Conv2D, Dropout
from tensorflow.keras.models import Model
from tensorflow.keras import Sequential
from tensorflow import keras
from tensorflow.keras import applications


from tqdm.notebook import tqdm
from tqdm import tqdm
import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
from datetime import datetime
import pytz
import json
from math import isclose, sqrt
from IPython import display


# !pip install twilio
# from twilio.rest import Client
def send_text(mess):#keep
    print('oops texting failed but the code goes on')


def save_obj(obj, name):#keep
    with open(name + '.pkl', 'wb') as f:
        pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)


def load_obj(name):#keep
    with open(name + '.pkl', 'rb') as f:
        return pickle.load(f)


def inds_sorted_data(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through 
  """
    log_ind = np.where(key_name == a['logs_names'])[0][0]
    val_list = a['all_logs'][:, log_ind]
    if max_or_min == 'max':
        max_arg_sort = np.flip(np.argsort(val_list)) + 1
    elif max_or_min == 'min':
        max_arg_sort = np.argsort(val_list) + 1
    else:
        raise ValueError("""max_or_min must be a string set to 'max' or 'min'""")
    return max_arg_sort


def sorted_loadable_epochs(a, key_name, max_or_min):#keep
    """
  a is the data form all_data looped through
  """
    arg_sort_inds = inds_sorted_data(data, key_name, max_or_min)
    arg_sort_inds[np.argmax(arg_sort_inds)] = -1
    saved_epoch_numbers = np.asarray(list(data['info']['epoch_dict'].keys()))
    sorted_loadable_epochs_out = []
    for k in arg_sort_inds:
        if k in saved_epoch_numbers:
            sorted_loadable_epochs_out.append(k)
    return sorted_loadable_epochs_out


def reload_info_dict(base_dir):
    info_dict_path = utils.get_files(base_dir, '*info_dict.json')[0]
    with open(info_dict_path, 'r') as f:
        D = json.load(f)
    # replace the directory to the main directory
    x = 'model_testing'
    old_base_dir = ''
    for k in D.keys():
        try:
            if x in D[k]:
                D[k] = base_dir.split(x)[0] + x + D[k].split(x)[-1]
                old_base_dir = D[k].split(x)[0]
        except:
            pass
    D['old_base_dir'] = old_base_dir
    D['checkpoints'] = utils.get_files(D['model_save_dir_checkpoints'], '*hdf5')
    D['label_num_names'] = D['label_key_name'].split('- (')[-1][:-1].split(', ')  # here use this to label the y limits
    D['label_nums'] = eval(D['label_key_name'].split(']')[0] + ']')

    checkpoint_epochs = []
    for k in D['checkpoints'][:-1]:
        checkpoint_epochs.append(int(k.split('_cp.hdf5')[0][-4:]))
    D['checkpoint_epochs'] = checkpoint_epochs

    epoch_dict = dict()
    for i, k in enumerate(checkpoint_epochs):
        epoch_dict[k] = D['checkpoints'][i]
    epoch_dict[-1] = D['checkpoints'][-1]
    D['epoch_dict'] = epoch_dict
    return D


def re_build_model(model_name_str, class_numbers, base_learning_rate=0.00001, dropout_val=None, IMG_SIZE=96,
                   labels=None):
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model


def load_model_data(all_models_directory):
    data_files = utils.get_files(all_models_directory, '*model_eval_each_epoch.json')
    to_split = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'
    all_data = []
    for i, k in enumerate(data_files):
        if '/small_h5s/' not in k:
            with open(k, 'r') as f:
                a = json.load(f)
            a['all_logs'] = np.asarray(a['all_logs'])
            new_log_names = []
            for k2 in a['logs_names']:
                new_log_names.append(k2.split('bool_')[-1])
            a['logs_names'] = new_log_names
            a['logs_names'] = np.asarray(a['logs_names'])

            k.split(to_split)[-1]
            a['full_name'] = '__'.join(' '.join(k.split(to_split)[-1].split('/2021')[0].split('_')).split('/'))
            a['dir'] = os.path.dirname(k)
            all_data.append(a)
    for i, k in enumerate(all_data):
        all_data[i]['info'] = reload_info_dict(k['dir'])
    return all_data




def build_model(info_dict, labels, model_name_str, base_learning_rate=0.00001, dropout_val=None, class_numbers=None,
                IMG_SIZE=96):
    if class_numbers is None:
        class_numbers = np.unique(labels)
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    base_model.trainable = True
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]
    for k in model.layers:
        k.trainable = False
    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
    class_weights = {i: rebalance[i] for i in class_numbers}
    wrap_vars_list = ['class_numbers',
                      'num_classes',
                      'base_learning_rate',
                      'model_name_str',
                      'IMG_SIZE',
                      'dropout_val']
    for k in wrap_vars_list:
        info_dict[k] = eval(k)
    info_dict['class_weights'] = str(class_weights)
    return model, class_weights, info_dict


# def label_naming_shorthand_dict(name_key=None):
#     label_naming_shorthand_dict = {
#         '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
#         '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
#         '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
#         '[0, 1]- (no touch, touch)': 'regular',
#         '[0, 1]- (not offset, offset)': 'only_offset',
#         '[0, 1]- (not onset, onset)': 'only_onset'}
#     if name_key is None:
#         return label_naming_shorthand_dict
#     else:
#         return label_naming_shorthand_dict[name_key]


# def info_dict_wrapper(info_dict, local_dict):
#     for k in local_dict.keys():
#         info_dict[k] = local_dict[k]
#     return info_dict


# def make_initial_folder(all_models_directory, unique_h5_train_val_dir):
#     single_frame_dir = all_models_directory + os.sep + unique_h5_train_val_dir + os.sep + 'data' + os.sep + 'single_frame'
#     Path(single_frame_dir).mkdir(parents=True, exist_ok=True)
#     return locals()
#     # return single_frame_dir


def get_automated_model_info_TL(BASE_H5, image_source_h5_directory_ending, test_data_dir, data_string_key="data"):#keep
    tz = pytz.timezone('America/Los_Angeles')
    loc_dt = pytz.utc.localize(datetime.utcnow())
    LA_TIME = loc_dt.astimezone(tz)
    todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")
    del tz
    del loc_dt
    del LA_TIME
    a = os.sep
    base_data_dir = BASE_H5 + a + data_string_key + a
    base_dir_all_h5s = BASE_H5 + a + data_string_key + a + 'single_frame' + a
    data_dir = base_data_dir + image_source_h5_directory_ending
    print('\nFOR IMAGES, 0 is train set, 1 is val set')
    print(data_dir)
    image_h5_list = utils.get_h5s(data_dir)
    # pdb.set_trace()
    h5_train = image_h5_list[0]
    h5_val = image_h5_list[1]
    # labels_dir = base_data_dir + a + "ALT_LABELS" + a
    # print('\nFOR LABELS,0 is train set, 1 is val set')
    # label_h5_list = utils.get_h5s(labels_dir)
    # print('\nSelect from the following label structures...')
    # print(labels_dir)
    # label_key_name_list = utils.print_h5_keys(label_h5_list[0], return_list=True)
    # h5_test_labels = utils.get_h5s(test_data_dir + a + "ALT_LABELS" + a, print_h5_list=False)[0]
    # h5_test = utils.get_h5s(test_data_dir + a + image_source_h5_directory_ending + a, print_h5_list=False)[0]
    return locals()


def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')
    return locals()


# get list of pre trained models to choose from
def get_keras_model_names():
    names_, types_ = utils.get_class_info(applications, return_name_and_type=True)
    model_names = np.asarray(names_)['function' == np.asarray(types_)]
    utils.print_list_with_inds(model_names)
    return model_names


# def make_model_save_directory(info_dict, make_folder=True):
#     naming_list = ['model_name_str', 'image_source_h5_directory_ending', 'label_key_shorthand', 'todays_version']
#     model_save_dir: str = copy.deepcopy(info_dict['BASE_H5'])
#     for k in naming_list:
#         model_save_dir += os.sep + info_dict[k] + os.sep
#     info_dict['model_save_dir'] = model_save_dir
#     info_dict['model_save_dir_checkpoints'] = model_save_dir + os.sep + 'checkpoints'
#     if make_folder:
#         Path(info_dict['model_save_dir_checkpoints']).mkdir(parents=True, exist_ok=True)
#     return model_save_dir


def basic_callbacks(save_checkpoint_filepath, monitor='val_loss', patience=10,
                    save_best_only=False, save_weights_only=True, save_freq="epoch", period=1):
    callbacks = []
    callbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, patience=patience))
    add_path_name = "{loss:.8f}_{epoch:04d}_cp.hdf5"
    add_path_name = "{epoch:04d}_cp.hdf5"
    callbacks.append(keras.callbacks.ModelCheckpoint(
        save_checkpoint_filepath + os.sep + add_path_name,
        monitor=monitor,
        save_best_only=save_best_only,
        save_weights_only=save_weights_only,
        save_freq=save_freq,
        period=period))
    return callbacks


def unzip_and_place_h5s(bd, do_delete_zips=False):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    shutil.copytree(bd, bd2)
    a = utils.get_files(bd2, '*.zip')
    for k in a:
        with zipfile.ZipFile(k, 'r') as zip_ref:
            zip_ref.extractall(os.path.dirname(k))
        if do_delete_zips:
            os.remove(k)


def change_to_local_dir(bd):
    bd2 = ''.join(bd.split('gdrive/My Drive'))
    return bd2


def change_to_gdrive_dir(bd):
    bd2 = '/content/gdrive/My Drive/'.join(bd.split('/content/'))
    return bd2


def pack_training_info(save_and_plot_history_var):
    training_info = dict()
    for k in ['L', 'all_logs', 'all_colors', 'logs_names', 'markers', 'matching_inds']:
        training_info[k] = eval('save_and_plot_history_var.' + k)
    for k in training_info.keys():
        if 'numpy.ndarray' in str(type(training_info[k])):
            training_info[k] = training_info[k].tolist()
    return training_info


def save_info_dict(info_dict_tmp):
    for k in info_dict_tmp.keys():
        if 'numpy.ndarray' in str(type(info_dict_tmp[k])):
            info_dict_tmp[k] = info_dict_tmp[k].tolist()
    with open(info_dict_tmp['model_save_dir'] + 'info_dict' + '.json', 'w') as f:
        json.dump(info_dict_tmp, f)


def foo_save_and_plot(training_info, save_and_plot_history_1, save_loc):
    text_str = transfer_learning.make_text_for_fig(training_info)
    training_info = transfer_learning.pack_training_info(training_info, save_and_plot_history_1)
    # save_num = foo_get_next_save_num(save_loc)

    # make figure and save
    transfer_learning.plot_train_hist(training_info, [1], [.9, 1], text_str)
    plt.savefig(save_loc + 'mod_test_fig_ACC' + '.png', bbox_inches="tight")
    transfer_learning.plot_train_hist(training_info, [0], [0, 0.25], text_str)
    plt.savefig(save_loc + 'mod_test_fig_LOSS' + '.png', bbox_inches="tight")

    # save training info
    with open(save_loc + 'model_eval_each_epoch' + '.json', 'w') as f:
        json.dump(training_info, f)


# In[17]:


# def foo_run_all():
#     # copy and unzip for colab
#     global all_models_directory, test_data_dir, info_dict, transfer_learning
#
#     local_dict = make_initial_folder(all_models_directory,
#                                      unique_h5_train_val_dir)  # make data single frame directory and get that directory
#     info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict
#
#     BASE_H5 = info_dict['all_models_directory'] + os.sep + info_dict[
#         'unique_h5_train_val_dir']  # directory for all the data for a certain type og images(lag or regular etc)
#
#     local_dict = get_automated_model_info(BASE_H5, image_source_h5_directory_ending,
#                                           test_data_dir)  # basic data like directories and train and val set (automated form the directory)
#     info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict
#
#     # """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#     local_dict = copy_over_new_labels(label_key_name, info_dict['image_h5_list'] + [info_dict['h5_test']],
#                                       info_dict['label_h5_list'] + [info_dict[
#                                                                         'h5_test_labels']])  # copy specific labels to the H5 of interest (this is all after the conversion from single frame)
#     info_dict = info_dict_wrapper(info_dict, local_dict)  # wrap output into a dict
#
#     # """$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$      make model     $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
#     # get all the labels
#     train_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_train']])
#     val_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_val']])
#     test_gen = image_tools.ImageBatchGenerator(batch_size, [info_dict['h5_test']])
#
#     train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']])
#     val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']])
#     labels = np.concatenate((train_labels, val_labels))
#
#     model, class_weights, info_dict = build_model(info_dict, labels, model_name_str,
#                                                   base_learning_rate=base_learning_rate,
#                                                   dropout_val=dropout_val)
#
#     #   info_dict['BASE_H5'] = change_to_gdrive_dir(info_dict['BASE_H5'])
#     model_save_dir = make_model_save_directory(
#         info_dict)  # make a unique folder using standard folder struct ending in a unique date/time folder
#     # ###change directory to gdrive in case it crashes
#     # dir_2_change = ['model_save_dir_checkpoints', 'model_save_dir']
#     # for k in dir_2_change:
#     #   info_dict[k] = change_to_gdrive_dir(info_dict[k])
#
#     model, class_weights, info_dict = build_model(info_dict,
#                                                   labels,
#                                                   model_name_str,
#                                                   base_learning_rate=base_learning_rate,
#                                                   dropout_val=dropout_val)
#
#     callbacks = basic_callbacks(info_dict['model_save_dir_checkpoints'], monitor=monitor, patience=patience,
#                                 save_best_only=save_best_only, save_weights_only=True,
#                                 save_freq=save_freq, period=period)
#
#     plot_callback = transfer_learning.save_and_plot_history(test_gen, key_name='labels', plot_metric_inds=[0])
#     callbacks.append(plot_callback)
#
#     ###FITTTTTTT
#     history = model.fit(train_gen,
#                         epochs=epochs,
#                         validation_data=val_gen,
#                         callbacks=callbacks,
#                         class_weight=class_weights)
#     # save finals checkpoint after model finishes
#     model.save_weights(info_dict['model_save_dir_checkpoints'] + os.sep + 'final_epoch_cp.hdf5')
#
#     training_info = pack_training_info(plot_callback)
#
#     xx = '/content/colab_data2/model_testing/all_data/all_models/small_h5s/InceptionV3/3lag/on-off_set_and_one_after/'
#     utils.get_files(xx, '*_cp.hdf5')
#
#     transfer_learning.foo_save_and_plot
#
#     foo_save_and_plot(training_info, plot_callback, info_dict['model_save_dir'])
#
#     save_info_dict(info_dict)




# def smooth(y, box_pts):#$%
#     box = np.ones(box_pts) / box_pts
#     y_smooth = np.convolve(y, box, mode='same')
#     return y_smooth


# def search_sequence_numpy(arr, seq): #$%
#     # Store sizes of input array and sequence
#     Na, Nseq = arr.size, seq.size
#
#     # Range of sequence
#     r_seq = np.arange(Nseq)
#
#     # Create a 2D array of sliding indices across the entire length of input array.
#     # Match up with the input sequence & get the matching starting indices.
#     M = (arr[np.arange(Na - Nseq + 1)[:, None] + r_seq] == seq).all(1)
#     # Get the range of those indices as final output
#     if M.any() > 0:
#         # return np.where(np.convolve(M,np.ones((Nseq),dtype=int))>0)[0]
#         return np.where(M)[0]
#     else:
#         return []  # No match found


# def search_sequence_numpy(arr, seq, return_type='indices'):#$%
#     Na, Nseq = arr.size, seq.size
#     r_seq = np.arange(Nseq)
#     M = (arr[np.arange(Na - Nseq + 1)[:, None] + r_seq] == seq).all(1)
#
#     if return_type == 'indices':
#         return np.where(M)[0]
#     elif return_type == 'bool':
#         return M


# def get_pairs_that_extend_out(num1, num2, array_in):#$%
#     a = np.where(array_in == num1)[0]
#     b = np.where(array_in == num2)[0]
#     c = sorted(np.hstack((a, b)))
#     c, _ = np.asarray(utils.group_consecutives(c))
#     bool_inds = np.zeros((3, len(c)), dtype=bool)
#     for i, k in enumerate(c):
#         N2 = array_in[k]
#         if np.all(N2 == num1):
#             bool_inds[0, i] = True
#         elif np.all(N2 == num2):
#             bool_inds[1, i] = True
#         else:
#             bool_inds[2, i] = True
#     return c, bool_inds


# def replotYlabels_everyX(D, every_x=10, len_plot=0):#$%
#     x = np.floor((len_plot - 1) / every_x)
#     for kk in range(int(x)):
#         kk += 1
#         for k, k2 in zip(D['label_nums'], D['label_num_names']):
#             plt.text(x=kk * every_x + .1, y=k, s=k2)


# class pole_plot():#$%
#     def __init__(self, img_h5_file, pred_val=None, true_val=None, threshold=0.5,
#                  len_plot=10, current_frame=0, figsize=[10, 5], label_nums=None, label_num_names=None, shift_by=0):
#         """
#         Examples
#         ________
#         a = analysis.pole_plot(
#             '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/master_train_test1.h5',
#             pred_val = [0,0,0,0,0,0,0,.2,.4,.5,.6,.7,.8,.8,.6,.4,.2,.1,0,0,0,0],
#             true_val = [0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
#             len_plot = 10)
#
#         a.plot_it()
#         """
#         if true_val is None:
#             if 'labels' in utils.print_h5_keys(img_h5_file, return_list=True, do_print=False):
#                 true_val = image_tools.get_h5_key_and_concatenate([img_h5_file])
#         tmp_loc = locals()
#         self.error_group_inds = None
#         if 'true_val' in tmp_loc and 'pred_val' in tmp_loc:
#             error_group_inds = np.where(true_val - pred_val != 0)[0]
#             error_group_inds, _ = utils.group_consecutives(error_group_inds)
#             self.error_group_inds = error_group_inds
#
#         self.isnotebook = utils.isnotebook()
#         self.img_h5_file = img_h5_file
#         self.pred_val = np.asarray(pred_val)
#         self.true_val = np.asarray(true_val)
#         self.threshold = threshold
#         self.len_plot = len_plot
#         self.current_frame = current_frame
#         self.figsize = figsize
#         self.fig_created = False
#         tmp3 = np.unique(np.concatenate((self.pred_val, self.true_val)))
#         self.range_labels = np.arange(np.nanmin(tmp3), np.nanmax(tmp3) + 1)
#         self.ylims = [np.nanmin(tmp3) - .5, np.nanmax(tmp3) + .5]
#
#         self.label_nums = label_nums
#         self.label_num_names = label_num_names
#         self.shift_by = shift_by
#         self.xlims = [-.5, self.len_plot - .5]
#         try:
#             self.pred_val_bool = (1 * (self.pred_val > threshold)).flatten()
#         except:
#             self.pred_val_bool = np.asarray(None)
#
#     def plot_it(self):
#
#         if self.fig_created is False or self.isnotebook:  # we need to create a new fig every time if we are in colab or jupyter
#             self.fig, self.axs = plt.subplots(2, figsize=self.figsize, gridspec_kw={'height_ratios': [1, 1]})
#             self.fig_created = True
#             # plt.subplots_adjust(hspace = .001)
#
#         ax1 = self.axs[1]
#         box = ax1.get_position()
#         box.y0 = box.y0 + self.shift_by
#         box.y1 = box.y1 + self.shift_by
#         ax1.set_position(box)
#
#         self.axs[0].clear()
#         self.axs[1].clear()
#         self.fig.suptitle('Touch prediction')
#         s1 = self.current_frame
#         s2 = self.current_frame + self.len_plot
#         self.xticks = np.arange(0, self.len_plot)
#         # plt.axis('off')
#         with h5py.File(self.img_h5_file, 'r') as h:
#             self.current_imgs = image_tools.img_unstacker(h['images'][s1:s2], s2 - s1)
#             # plt.imshow(self.current_imgs)
#             self.axs[0].imshow(self.current_imgs)
#             self.axs[0].axis('off')
#
#         leg = []
#         # axs[1].plot([None])
#         if len(self.pred_val.shape) != 0:
#             plt.plot(self.pred_val[s1:s2].flatten(), color='black', marker='*', linestyle='None')
#             leg.append('pred')
#         # if len(self.pred_val_bool.shape) != 0:
#         #     plt.plot(self.pred_val_bool[s1:s2].flatten(), '.g', markersize=10)
#         #     leg.append('bool_pred')
#         if len(self.true_val.shape) != 0:
#             tmp1 = self.true_val[s1:s2].flatten()
#             plt.scatter(range(len(tmp1)), tmp1, s=80, facecolors='none', edgecolors='r')
#             leg.append('actual')
#         if leg:
#             plt.legend(leg, bbox_to_anchor=(1.04, 1), borderaxespad=0)
#             plt.ylim(self.ylims)
#             plt.xlim(self.xlims)
#             _ = plt.xticks(ticks=self.xticks)
#         if self.label_nums is not None and self.label_num_names is not None:
#             plt.yticks(ticks=self.label_nums, labels=self.label_num_names)
#         plt.grid(axis='y')
#
#     def next(self):
#         self.current_frame = self.current_frame + self.len_plot
#         self.plot_it()
#
#     def move(self, move_val):
#         self.current_frame = self.current_frame + move_val
#         self.plot_it()


# def get_class_info(c, sort_by_type=True, include_underscore_vars=False, return_name_and_type=False, end_prev_len=40):#$%
#     names = []
#     type_to_print = []
#     for k in dir(c):
#         if include_underscore_vars is False and k[0] != '_':
#             tmp1 = str(type(eval('c.' + k)))
#             type_to_print.append(tmp1.split("""'""")[-2])
#             names.append(k)
#         elif include_underscore_vars:
#             tmp1 = str(type(eval('c.' + k)))
#             type_to_print.append(tmp1.split("""'""")[-2])
#             names.append(k)
#     len_space = ' ' * max(len(k) for k in names)
#     len_space_type = ' ' * max(len(k) for k in type_to_print)
#     if sort_by_type:
#         ind_array = np.argsort(type_to_print)
#     else:
#         ind_array = np.argsort(names)
#
#     for i in ind_array:
#         k1 = names[i]
#         k2 = type_to_print[i]
#         # k3 = str(c[names[i]])
#         k3 = str(eval('c.' + names[i]))
#         # [-40:]
#         k1 = (k1 + len_space)[:len(len_space)]
#         k2 = (k2 + len_space_type)[:len(len_space_type)]
#         if len(k3) > end_prev_len:
#             k3 = '...' + k3[-end_prev_len:]
#         else:
#             k3 = '> ' + k3[-end_prev_len:]
#
#         print(k1 + ' type->   ' + k2 + '  ' + k3)
#     if return_name_and_type:
#         return names, type_to_print


# def get_dict_info(c, sort_by_type=True, include_underscore_vars=False, return_name_and_type=False, end_prev_len=40):#$%
#     names = []
#     type_to_print = []
#     for k in c.keys():
#         if include_underscore_vars is False and k[0] != '_':
#             tmp1 = str(type(c[k]))
#             type_to_print.append(tmp1.split("""'""")[-2])
#             names.append(k)
#         elif include_underscore_vars:
#             tmp1 = str(type(c[k]))
#             type_to_print.append(tmp1.split("""'""")[-2])
#             names.append(k)
#     len_space = ' ' * max(len(k) for k in names)
#     len_space_type = ' ' * max(len(k) for k in type_to_print)
#     if sort_by_type:
#         ind_array = np.argsort(type_to_print)
#     else:
#         ind_array = np.argsort(names)
#
#     for i in ind_array:
#         k1 = names[i]
#         k2 = type_to_print[i]
#         k3 = str(c[names[i]])
#
#         k1 = (k1 + len_space)[:len(len_space)]
#         k2 = (k2 + len_space_type)[:len(len_space_type)]
#         if len(k3) > end_prev_len:
#             k3 = '...' + k3[-end_prev_len:]
#         else:
#             k3 = '> ' + k3[-end_prev_len:]
#
#         print(k1 + ' type->   ' + k2 + '  ' + k3)
#     if return_name_and_type:
#         return names, type_to_print


"""# download the retrain augmented and OG files to local """


def foo_rename(instr):#keep
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


"""# download the retrain augmented and OG files to local """
train_and_val_dir = '/Users/phil/Desktop/content/ALL_RETRAIN_H5_data____temp'
srs_TandV_h5s, dst_TandV_h5s = utils.copy_file_filter(
    foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/small_h5s/data'),
    train_and_val_dir,
    keep_strings='/3lag/',
    remove_string='.zip',
    overwrite=True,
    just_print_what_will_be_copied=False, return_list_of_files=True)

h5_imgs, h5_labels, inds_of_files = utils.copy_alt_labels_based_on_directory(srs_TandV_h5s)
for i in inds_of_files:
    utils.copy_over_all_non_image_keys(h5_labels[i], dst_TandV_h5s[i])

base_test_dir = '/Users/phil/Desktop/content/DATA_FULL____temp'
srs_test_h5s, dst_test_h5s = utils.copy_file_filter(
    foo_rename('/content/gdrive/My Drive/colab_data2/model_testing/all_data/test_data/small_h5s'),
    base_test_dir,
    keep_strings='/3lag/',
    remove_string='.zip',
    overwrite=True,
    just_print_what_will_be_copied=False, return_list_of_files=True)

h5_imgs, h5_labels, inds_of_files = utils.copy_alt_labels_based_on_directory(srs_test_h5s)
for i in inds_of_files:
    utils.copy_over_all_non_image_keys(h5_labels[i], dst_test_h5s[i])

"""#trasnform h5s to LSTM H5s"""

base_dir = '/Users/phil/Desktop/content/'
for h5 in utils.lister_it(utils.get_h5s(base_dir), keep_strings='____temp'):  # only go through the non converted h5s
    # utils.print_h5_keys(h5)
    h5_out = ''.join(h5.split('____temp'))
    print(h5, h5_out)
    image_tools.convert_h5_to_LSTM_h5(h5, h5_out)
    utils.copy_over_all_non_image_keys(h5, h5_out)

train_and_val_dir = ''.join(train_and_val_dir.split('____temp')) + os.sep
base_test_dir = ''.join(base_test_dir.split('____temp')) + os.sep
"""#copy the alt labels to the h5 files """

"""#working TL"""


# bd2 = "/content/gdrive/My Drive/colab_data2/"
# all_models_directory = bd2+"/model_testing/all_data/all_models/"
# all_data = load_model_data(all_models_directory)


# save_obj(all_data, '/content/all_data2')
all_data = load_obj('/Users/phil/Downloads/all_data')

"""# code for model TMP TMP """

class save_and_plot_history(keras.callbacks.Callback):
    def __init__(self, my_test_batch_generator, key_name='labels', plot_metric_inds=[0]):
        self.all_y = image_tools.get_h5_key_and_concatenate(my_test_batch_generator.H5_file_list, key_name=key_name)
        self.my_test_batch_generator = my_test_batch_generator
        self.all_logs = np.asarray([])
        self.markers = np.tile(['-', '--', ':'], len(plot_metric_inds))
        c = plt.rcParams["axes.prop_cycle"].by_key()["color"]
        self.all_colors = c
        self.color = np.repeat([c[k] for k in plot_metric_inds], 3)
        self.plot_metric_inds = plot_metric_inds

    def on_epoch_end(self, epoch, logs=None):
        if epoch == 0 or 'logs_names' not in dir(self):

            self.L = len(self.model.metrics_names)
            L = len(self.model.metrics_names)
            self.matching_inds = np.reshape(np.linspace(0, L * 3 - 1, L * 3), [3, -1]).astype('int')
            self.plot_metric_inds = self.plot_metric_inds + [k + L for k in self.plot_metric_inds] + [k + L * 2 for k in
                                                                                                      self.plot_metric_inds]
            self.plot_metric_inds = np.swapaxes(np.reshape(self.plot_metric_inds, [3, -1]), 1, 0).flatten()
            self.fig = plt.figure(self.plot_metric_inds[0], figsize=[15, 10])
            self.logs_names = self.model.metrics_names + [k + '_val' for k in self.model.metrics_names] + [k + '_test'
                                                                                                           for k in
                                                                                                           self.model.metrics_names]

            self.logs_names_selected = []
            plt.ylim([.95, 1])
            for k in self.plot_metric_inds:
                self.logs_names_selected.append(self.logs_names[k])
        if logs is not None:
            log_list = []
            _ = [log_list.append(logs[k]) for k in logs.keys()]
            logs_test = self.model.evaluate(self.my_test_batch_generator)
            log_list = log_list + logs_test
            if len(self.all_logs) != 0:
                self.all_logs = np.vstack((self.all_logs, np.asarray(log_list)))
            else:
                self.all_logs = np.concatenate((self.all_logs, np.asarray(log_list)), axis=0)
            # if epoch > 0:
            if len(self.all_logs.shape) > 1:
                display.clear_output(wait=True)
                for kmark, kcol, k in zip(self.markers, self.color, self.plot_metric_inds):
                    plt.plot(self.all_logs[:, k], color=kcol, linestyle=kmark)

                plt.legend(self.logs_names_selected)
                plt.grid(True)
                display.display(plt.gcf())


def save_info_dict(info_dict_tmp):  # added os.sep to saveing json file
    for k in info_dict_tmp.keys():
        if 'numpy.ndarray' in str(type(info_dict_tmp[k])):
            info_dict_tmp[k] = info_dict_tmp[k].tolist()
    with open(info_dict_tmp['model_save_dir'] + os.sep + 'info_dict' + '.json', 'w') as f:
        json.dump(info_dict_tmp, f)


# def foo_training_validation_type_controller(ind_number, target_path="/content/ALL_RETRAIN_H5_data"):
#     tmp1 = utils.get_files(target_path, '*.hh55')
#     for k in tmp1:
#         os.rename(k, k.split('.hh55')[0] + '.h5')
#
#     b = utils.get_h5s(target_path, 0)
#     if ind_number == 0:
#         # KEEP train_AUG and val_REG
#         remove_list = list(utils.lister_it(b, remove_string=['_AUG.h5', 'val']))
#         remove_list = remove_list + list(utils.lister_it(b, keep_strings='AUG', remove_string=['train']))
#         for k in remove_list:
#             print(k)
#             os.rename(k, k.split('.h5')[0] + '.hh55')
#     elif ind_number == 1:
#         # KEEP train_AUG and val_AUG
#         remove_list = list(utils.lister_it(b, remove_string=['_AUG.h5']))
#         print(remove_list)
#         for k in remove_list:
#             print(k)
#             os.rename(k, k.split('.h5')[0] + '.hh55')
#     elif ind_number == 2:
#         # KEEP train_REG and val_REG
#         remove_list = list(utils.lister_it(b, keep_strings=['_AUG.h5']))
#         print(remove_list)
#         for k in remove_list:
#             print(k)
#             os.rename(k, k.split('.h5')[0] + '.hh55')


def re_build_model_TL(model_name_str, class_numbers, base_learning_rate=0.00001,
                      dropout_val=None, IMG_SIZE=96, labels=None, reload_weights_file=None, num_layers_unfreeze=0):#keep
    num_classes = len(class_numbers)
    IMG_SHAPE = (IMG_SIZE, IMG_SIZE, 3)
    model_function = eval('applications.' + model_name_str)
    base_model = model_function(input_shape=IMG_SHAPE, weights='imagenet', include_top=False)
    num_layers_in_base_model = len(base_model.layers)
    base_model_layer_names = [k.name for k in base_model.layers]
    base_model.trainable = False  ##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$##&%&$&$&%&%&$&%&$&$&%&%&$&&$&&%&%&$&&$&$&&&$
    len_base_model = len(base_model.layers)
    x = base_model.output
    x = keras.layers.GlobalAveragePooling2D()(x)  # global spatial average pooling layer
    x = Dense(2048, activation='relu')(x)  # fully-connected layer
    if dropout_val is not None:
        x = Dropout(dropout_val)(x)
    ###### i need to name the layers
    if num_classes == 2:
        predictions = Dense(1, activation='sigmoid')(x)  # fully connected output/classification layer
    else:
        predictions = Dense(num_classes, activation='softmax')(x)  # fully connected output/classification layer
    model = Model(inputs=base_model.input, outputs=predictions)

    if num_classes == 2:

        optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
        loss = keras.losses.BinaryCrossentropy()
        metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5),
                   keras.metrics.AUC(name='auc')]
    else:
        optimizer = keras.optimizers.Adam(learning_rate=base_learning_rate)
        loss = keras.losses.SparseCategoricalCrossentropy()
        metrics = [keras.metrics.SparseCategoricalAccuracy(name='acc')]
    if reload_weights_file is not None:
        model.load_weights(reload_weights_file)  # load model weights

    # for i, k in enumerate(model.layers):
    #   if i >= len_base_model-5:
    #     k.trainable = True
    #   else:
    #     k.trainable = False
    relu_layers = []
    for i, k in enumerate(model.layers):
        if 'relu' in k.name.lower():
            relu_layers.append(i)
    relu_layers.append(9999999)
    # relu_layers = np.flip(np.asarray(relu_layers)+1)
    relu_layers = np.flip(np.asarray(relu_layers))

    # num_layers_unfreeze =1          #0 means freeze entire model 1 means base model forzen 2 one more (group)laye runfrozen etc
    for i, k in enumerate(model.layers):
        if i >= relu_layers[
            num_layers_unfreeze] and 'batchnorm' not in k.name.lower():  # LK:J:LKJD:LKJD:LKDJ:LDKJ:DLKJD:LKJD:LKJD:LKJD:LKDJ:LKDJLK:DJ
            k.trainable = True
        else:
            k.trainable = False

    model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    model.num_layers_in_base_model = num_layers_in_base_model
    model.base_model_layer_names = base_model_layer_names
    if labels is not None:
        rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
        class_weights = {i: rebalance[i] for i in class_numbers}
        wrap_vars_list = ['class_numbers',
                          'num_classes',
                          'base_learning_rate',
                          'model_name_str',
                          'IMG_SIZE',
                          'dropout_val']
        return model, class_weights
    else:
        return model


for TMP_basename in next(os.walk(train_and_val_dir))[1]:
    if 'ipynb_checkpoints' not in TMP_basename:
        print(TMP_basename)
# righthere2
"""##train    asdfasfasdfasfasdfasdfasdf"""


def foo_rename2(instr):#keep
    if isinstance(instr, str) and '/My Drive/' in instr:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]
    else:
        return instr


def make_D_local(D):#keep
    for key in D:
        if isinstance(D[key], str):
            D[key] = foo_rename2(D[key])
    for key in D['epoch_dict']:
        if isinstance(D['epoch_dict'][key], str):
            D['epoch_dict'][key] = foo_rename2(D['epoch_dict'][key])
    return D


def add_lstm_to_model(model_in, num_layers_in_base_model, base_learning_rate=10 ** -5, lstm_len=7):#keep
    # model_in.base_model_layer_names
    base_model = Model(model_in.input, model_in.layers[num_layers_in_base_model - 1].output)
    model_out = Sequential()
    model_out.add(TimeDistributed(base_model, input_shape=(lstm_len, 96, 96, 3)))
    model_out.add(TimeDistributed(Flatten()))
    model_out.add(LSTM(256, activation='relu', return_sequences=False))
    model_out.add(Dense(64, activation='relu'))
    # model_out.add(Dropout(.5))
    model_out.add(Dense(1, activation='sigmoid'))

    optimizer = keras.optimizers.RMSprop(learning_rate=base_learning_rate)
    loss = keras.losses.BinaryCrossentropy()
    metrics = [keras.metrics.BinaryAccuracy(name="bool_acc", threshold=0.5), keras.metrics.AUC(name='auc')]
    model_out.compile(optimizer=optimizer, loss=loss, metrics=metrics)
    return model_out


def rename_content_folder(instr):# keep
    if isinstance(instr, str) and '/content/' in instr:
        return '/Users/phil/Desktop/content' + instr.split('content')[-1]
    else:
        return instr


# train_and_val_dir
# base_test_dir
key_name = 'acc_test'
max_or_min = 'max'
models_2_load = [45]
# $%^&&&& user hard coded settings
patience = 8  # DETERMINES early stopping
save_freq = "epoch"  # leave this as epoch
period = 1  # DETERMINES how often it saves the checkpoints
epochs = 12  # DETERMINES how many epochs the model trains for if early stopping is never triggered
batch_size = 2 ** 7  # DETERMINES number of images per batch
save_best_only = True
monitor = 'val_loss'
###

lstm_len = 7
send_text('training started, you will receive a text when all training is done')

num_layers_to_unfreeze_list = [0]
for num_layers_unfreeze in num_layers_to_unfreeze_list:  # in the resnet base model
    # all_models_directory =  copy.deepcopy(train_and_val_dir)   # DETERMINES location for all model you will run #del
    # FOR EACH UNIQUE BASE DATSET start
    for TMP_basename in next(os.walk(train_and_val_dir))[1]:
        test_data_dir = base_test_dir + os.sep + TMP_basename  # # doesnt determine anything just filler, I define test data based on variable "test_path"
        unique_h5_train_val_dir = copy.deepcopy(
            TMP_basename)  # ^^^^^^^DETERMINES^^^^^^^ the name of the folder where each type of data is stored
        # FOR EACH UNIQUE BASE DATSET end
        for iiii, model_ind in enumerate(tqdm(models_2_load)):
            if 'ipynb_checkpoints' not in TMP_basename:
                data = all_data[model_ind]
                best_epochs = sorted_loadable_epochs(data, key_name, max_or_min)
                reload_model_epoch = best_epochs[0]
                D = make_D_local(copy.deepcopy(data['info']))
                test_path = utils.get_h5s(base_test_dir + os.sep + TMP_basename, 0)[0]

                LEARNING_RATE = D['base_learning_rate']
                # LEARNING_RATE = LEARNING_RATE/2
                LEARNING_RATE = 10 ** -5
                dropout_val = .1
                # righthere4
                model = re_build_model_TL(D['model_name_str'], D['class_numbers'],
                                          base_learning_rate=LEARNING_RATE,
                                          dropout_val=dropout_val, IMG_SIZE=D['IMG_SIZE'],
                                          reload_weights_file=D['epoch_dict'][reload_model_epoch],
                                          num_layers_unfreeze=num_layers_unfreeze)

                model = add_lstm_to_model(model, model.num_layers_in_base_model, base_learning_rate=LEARNING_RATE,
                                          lstm_len=lstm_len)

                # def add_LSTM_to_model(model, num_layers_in_base_model, base_model_layer_names):
                """
        image_tools.ImageBatchGenerator_simple(batch_size, [h5s], label_key = 'labels')
        3) ensure saving is correct saving the model to local and the moving it to its final destination, 
        4) ensure training val and test are using the correct simple generator (maybe dont need the test data??????? likely do though)
        5) copy all the alt labels to the H5s automatically  
        """

                label_ind = np.where(np.asarray(D['label_key_name']) == np.asarray(
                    list(model_maker.label_naming_shorthand_dict().keys())))[0][0]
                pred_key_save_name = 'MODEL_2_' + data[
                    'full_name'] + '__' + key_name + ' ' + max_or_min + '__epoch ' + str(
                    reload_model_epoch) + '__L_ind' + str(label_ind) + '__LABELS'
                print(pred_key_save_name)

                # FOR EACH UNIQUE MODEL start
                image_source_h5_directory_ending = D[
                    'image_source_h5_directory_ending']  # ^^^^^^^DETERMINES^^^^^^^ THE IMAGE SOURCE
                label_key = D[
                    'label_key_name']  # ^^^^^^^DETERMINES^^^^^^^ THE LABEL SOURCE choose the ind based on the print out
                # label_key = 'labels'#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                tz = pytz.timezone('America/Los_Angeles')
                loc_dt = pytz.utc.localize(datetime.utcnow())
                LA_TIME = loc_dt.astimezone(tz)
                todays_version = LA_TIME.strftime("%Y_%m_%d_%H_%M_%S")

                a = D['model_save_dir_checkpoints']
                b = '/'.join([k for k in a.split('/') if k != ''])
                b = b.split('all_models/')[-1].split('/')[1:]

                base_data = unique_h5_train_val_dir.split('data_')[-1]
                b.insert(4, todays_version)
                # b.insert(0, type_trained_on)
                b.insert(0, base_data)
                new_base_folder_name = "LSTM_mod_" + str(model_ind) + "_" + str(num_layers_unfreeze)
                b.insert(0, new_base_folder_name)  ##################
                end_save_dir = os.sep.join(b)
                model_save_dir_checkpoints = rename_content_folder('/content/' + end_save_dir)

                ################## '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'+end_save_dir

                Path(model_save_dir_checkpoints).mkdir(parents=True, exist_ok=True)

                # info_dict = get_automated_model_info(train_and_val_dir, image_source_h5_directory_ending, test_data_dir, data_string_key = unique_h5_train_val_dir)
                info_dict = get_automated_model_info_TL(train_and_val_dir, image_source_h5_directory_ending, test_data_dir,
                                                        data_string_key='')

                # val_gen = image_tools.ImageBatchGenerator(batch_size, info_dict['h5_val'], label_key = label_key)
                val_gen = image_tools.ImageBatchGenerator_simple(batch_size, info_dict['h5_val'], label_key=label_key)
                # train_gen = image_tools.ImageBatchGenerator(batch_size, info_dict['h5_train'], label_key = label_key)
                train_gen = image_tools.ImageBatchGenerator_simple(batch_size, info_dict['h5_train'],
                                                                   label_key=label_key)
                # test_path = info_dict['h5_val']#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                # test_gen = image_tools.ImageBatchGenerator(batch_size, test_path, label_key = label_key)
                test_gen = image_tools.ImageBatchGenerator_simple(batch_size, test_path, label_key=label_key)
                # test_gen = val_gen#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                # test_gen = image_tools.ImageBatchGenerator(batch_size, '/content/small_test.h5', label_key = label_key)#TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                # FOR EACH UNIQUE MODEL end

                train_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_train']], label_key)
                val_labels = image_tools.get_h5_key_and_concatenate([info_dict['h5_val']], label_key)
                labels = np.concatenate(
                    (train_labels, val_labels))  # flkjasklfjklasjdflkjaslkfjalksdjflasdfkasdfjalsdfkjasdflk
                labels = train_labels
                class_numbers = np.unique(labels)  #### can use label info here but no need
                rebalance = class_weight.compute_class_weight('balanced', classes=class_numbers, y=labels.flatten())
                class_weights = {i: rebalance[i] for i in class_numbers}

                callbacks = basic_callbacks(model_save_dir_checkpoints, monitor=monitor, patience=patience,
                                            save_best_only=save_best_only, save_weights_only=True,
                                            save_freq=save_freq, period=period)

                plot_callback = transfer_learning.save_and_plot_history(test_gen, key_name=label_key, plot_metric_inds=[
                    1])  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                plot_callback = save_and_plot_history(test_gen, key_name=label_key, plot_metric_inds=[
                    1])  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_
                callbacks.append(
                    plot_callback)  # TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_TEMP_CHANGE_

                ###FITTTTTTT

                print(D['image_source_h5_directory_ending'], label_key)
                history = model.fit(train_gen,
                                    epochs=epochs,
                                    validation_data=val_gen,
                                    class_weight=class_weights,
                                    callbacks=callbacks)

                for k in info_dict.keys():
                    D[k] = info_dict[k]

                training_info = pack_training_info(plot_callback)
                foo_save_and_plot(training_info, plot_callback, os.path.dirname(model_save_dir_checkpoints) + os.sep)

                # for saving purposed
                D['base_learning_rate'] = LEARNING_RATE
                D['num_layers_unfreeze'] = num_layers_unfreeze
                D['dropout_val'] = dropout_val

                base_dir = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/all_models/'

                model_save_dir_checkpoints_Gdrive = copy.deepcopy(model_save_dir_checkpoints)
                model_save_dir_checkpoints_Gdrive = base_dir + model_save_dir_checkpoints_Gdrive.split('/content')[-1]
                D['model_save_dir_checkpoints'] = model_save_dir_checkpoints_Gdrive
                D['model_save_dir'] = os.path.dirname(model_save_dir_checkpoints_Gdrive)

                # once finished transfer the completed trained model to the final directory in gdrive
                target_path = os.path.dirname(base_dir + os.sep + end_save_dir)
                Path(target_path).mkdir(parents=True, exist_ok=True)
                save_info_dict(D)  # save file after folder is created
                sync(os.path.dirname(model_save_dir_checkpoints), target_path, 'sync')

                shutil.rmtree(os.path.dirname(model_save_dir_checkpoints))

send_text('finished with all of it')



from whacc import PoleTracking, utils
import whacc
import glob
import os

mp4_path = "/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here_FINISHED/"
mp4_path = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/'
folders_with_MP4s = whacc.utils.recursive_dir_finder(mp4_path, '*.mp4')
_ = [print(str(i) + ' ' + k) for i, k in enumerate(folders_with_MP4s)]

for video_directory in folders_with_MP4s:
    utils.make_mp4_list_dict(video_directory)

you_are_samson_king = True
if you_are_samson_king:
  CROP_SIZE = [71, 71]
else:
  CROP_SIZE = [61, 61]

init_template_dir = None
PT = dict()
for i, video_directory in enumerate(folders_with_MP4s):
    template_img_full_name = glob.glob(video_directory + os.path.sep + '*template_img.png')
    if len(template_img_full_name) == 1:
        init_template_dir = template_img_full_name
    if len(template_img_full_name) == 0:  # if the template already exists
        PT[i] = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
        PT[i].custom_init_pole_template_dir = init_template_dir # auto find a close match when init-ing
        PT[i].cut_out_pole_template(crop_size=CROP_SIZE, frame_num=2000, file_ind=2)  # user cut out template image
        PT[i].save_template_img(cust_save_dir=PT[i].video_directory)  # save the template image



PT[i] = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
init_template_dir = '/Users/phil/Desktop/template_img_set_test.png'
PT[i].custom_init_pole_template_dir = init_template_dir # auto find a close match when init-ing
PT[i].cut_out_pole_template(crop_size=CROP_SIZE, frame_num=2000, file_ind=2)  # user cut out template image
PT[i].save_template_img(cust_save_dir=PT[i].video_directory)  # save the template image



# PT = PoleTracking(video_directory=video_directory)  # create the class
# PT.cut_out_pole_template(crop_size=CROP_SIZE, frame_num=2000, file_ind=0)
# PT.save_template_img(cust_save_dir=PT.video_directory)
#
#
#
#
# you_are_samson_king = True
# if you_are_samson_king:
#   CROP_SIZE = [71, 71]
# else:
#   CROP_SIZE = [61, 61]
#
# init_template_dir = None
# PT = dict()
# for i, video_directory in enumerate(folders_with_MP4s):
#     template_img_full_name = glob.glob(video_directory + os.path.sep + '*template_img.png')
#     init_template_dir = template_img_full_name
#     if len(template_img_full_name) == 0:  # if the template already exists
#         PT[i].custom_init_pole_template_dir = init_template_dir # auto find a close match when init-ing
#         PT[i] = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
#         PT[i].cut_out_pole_template(crop_size=CROP_SIZE, frame_num=2000, file_ind=2)  # user cut out template image
#         PT[i].save_template_img(cust_save_dir=PT[i].video_directory)  # save the template image
#
#



# from whacc.image_tools import *
# import copy
# import pdb
#
#
# class ImageBatchGenerator_feature_array(keras.utils.Sequence):
#
#     def __init__(self, time_length, batch_size, h5_file_list, label_key='labels', feature_len=None,
#                  label_index_to_lstm_len=None, edge_value=-1, remove_any_time_points_with_edges = True):
#         """
#
#         Parameters
#         ----------
#         time_length : total time points
#         batch_size : batch output for generator
#         h5_file_list : list of h5 strings or single h5 string
#         label_key : where y output comes from
#         feature_len : length of the features per time point
#         label_index_to_lstm_len : determines look back and look forward index refers to where the 'current' time point is
#         within the range of look_back_len; e.g. look_back_len = 7 label_index_to_lstm_len = 3 (middle index of 7) then
#         time point 0 will be at 3 and index 0, 1, 2 will be the past values and index 4, 5, 6 will be the future values.
#         look_back_len = 7 label_index_to_lstm_len = 0 (first index) then current time point will be at index 0 and all
#         other time point (1, 2, 3, 4, 5, 6) will be future values. Default is middle time point
#         edge_value : what to replace the edge values with, when time shifting you will have edges with no value, this
#         will replace those values with this number.
#         remove_any_time_points_with_edges : if true then batch size will not be the actual batch size it will be batch
#         size - the number of time points with edges in them, x and y will still match and this method is preferred for
#         training due to it not including unknown values.
#         """
#         assert time_length % 2 == 1, "number of images must be odd"
#         if label_index_to_lstm_len is None:
#             label_index_to_lstm_len = time_length // 2  # in the middle
#         h5_file_list = utils.make_list(h5_file_list, suppress_warning=True)
#         num_frames_in_all_H5_files = get_total_frame_count(h5_file_list)
#         file_inds_for_H5_extraction = batch_size_file_ind_selector(
#             num_frames_in_all_H5_files, batch_size)
#         subtract_for_index = reset_to_first_frame_for_each_file_ind(
#             file_inds_for_H5_extraction)
#         self.remove_any_time_points_with_edges = remove_any_time_points_with_edges
#         self.label_key = label_key
#         self.batch_size = batch_size
#         self.H5_file_list = h5_file_list
#         self.num_frames_in_all_H5_files = num_frames_in_all_H5_files
#         self.file_inds_for_H5_extraction = file_inds_for_H5_extraction
#         self.subtract_for_index = subtract_for_index
#         self.label_index_to_lstm_len = label_index_to_lstm_len
#         self.lstm_len = time_length
#         self.feature_len = feature_len
#         self.edge_value = edge_value
#         if remove_any_time_points_with_edges:
#             self.edge_value = np.nan
#             print('remove_any_time_points_with_edges == True : forcing edge_value to np.nan to aid in removing these time points')
#
#
#         self.get_frame_edges()
#         # self.full_edges_mask = self.full_edges_mask - (self.lstm_len // 2 - self.label_index_to_lstm_len)
#
#     def __getitem__(self, num_2_extract):
#         h = self.H5_file_list
#         i = self.file_inds_for_H5_extraction
#         all_edges = self.all_edges_list[np.int(i[num_2_extract])]
#         H5_file = h[np.int(i[num_2_extract])]
#         num_2_extract_mod = num_2_extract - self.subtract_for_index[num_2_extract]
#
#         with h5py.File(H5_file, 'r') as h:
#             b = self.lstm_len // 2
#             tot_len = h['images'].shape[0]
#
#             # assert tot_len - b > self.batch_size, "reduce batch size to be less than total length of images minus floor(lstm_len) - 1, MAX->" + str(
#             #     tot_len - b - 1)
#
#             i1 = num_2_extract_mod * self.batch_size - b
#             i2 = num_2_extract_mod * self.batch_size + self.batch_size + b
#             edge_left_trigger = abs(min(i1, 0))
#             edge_right_trigger = min(abs(min(tot_len - i2, 0)), b)
#             x = h['images'][max(i1, 0):min(i2, tot_len)]
#             if edge_left_trigger + edge_right_trigger > 0:  # in case of edge cases
#                 pad_shape = list(x.shape)
#                 pad_shape[0] = edge_left_trigger + edge_right_trigger
#                 pad = np.zeros(pad_shape).astype('float32')
#                 if edge_left_trigger > edge_right_trigger:
#                     x = np.concatenate((pad, x), axis=0)
#                 else:
#                     x = np.concatenate((x, pad), axis=0)
#
#             s = list(x.shape)
#             s.insert(1, self.lstm_len)
#             out = np.zeros(s).astype('float32')  # before was uint8
#             Z = self.label_index_to_lstm_len - self.lstm_len // 2
#             for i in range(self.lstm_len):
#                 i_temp = i
#                 i = i - Z
#                 i1 = max(0, b - i)
#                 i2 = min(s[0], s[0] + b - i)
#                 i3 = max(0, i - b)
#                 i4 = min(s[0], s[0] + i - b)
#                 # print('take ', i3, ' to ', i4, ' and place in ', i1, ' to ', i2)
#                 out[i1:i2, i_temp, ...] = x[i3:i4, ...]
#
#             out = out[b:s[0] - b, ...]
#             i1 = num_2_extract_mod * self.batch_size
#             i2 = num_2_extract_mod * self.batch_size + self.batch_size
#             raw_Y = h[self.label_key][i1:i2]
#
#             adjust_these_edge_frames = np.intersect1d(all_edges.flatten(), np.arange(i1, i2))
#             b2 = b - self.label_index_to_lstm_len  # used to adjust mask postion based on where the center value is
#             for atef in adjust_these_edge_frames:
#                 mask_ind = np.where(atef == all_edges)[1][0]
#                 mask_ind = mask_ind - b2
#                 mask_ind = mask_ind % self.full_edges_mask.shape[0]  # wrap around index
#
#                 mask_ = self.full_edges_mask[mask_ind]
#                 mask_ = mask_ == 1
#                 out_ind = atef + i1 - b2
#                 out_ind = out_ind % out.shape[0]  # wrap around index
#                 out[out_ind][mask_] = self.edge_value
#
#
#             s = out.shape
#             out = np.reshape(out, (s[0], s[1] * s[2]))
#             if self.remove_any_time_points_with_edges:
#                 keep_inds = ~np.isnan(np.mean(out, axis = 1))
#                 out = out[keep_inds]
#                 raw_Y = raw_Y[keep_inds]
#
#             return out, raw_Y
#
#     def __len__(self):
#         return len(self.file_inds_for_H5_extraction)
#
#     def getXandY(self, num_2_extract):
#         """
#
#         Parameters
#         ----------
#         num_2_extract :
#
#
#         Returns
#         -------
#
#         """
#         rgb_tensor, raw_Y = self.__getitem__(num_2_extract)
#         return rgb_tensor, raw_Y
#
#     def image_transform(self, raw_X):
#         """input num_of_images x H x W, image input must be grayscale
#         MobileNetV2 requires certain image dimensions
#         We use N x 61 x 61 formated images
#         self.IMG_SIZE is a single number to change the images into, images must be square
#
#         Parameters
#         ----------
#         raw_X :
#
#
#         Returns
#         -------
#
#
#         """
#         # kept this cause this is the format of the image generators I know this is redundant
#         rgb_batch = copy.deepcopy(raw_X)
#         rgb_tensor = rgb_batch
#         self.IMG_SHAPE = (self.feature_len)
#         return rgb_tensor
#
#     def get_frame_edges(self):
#         self.all_edges_list = []
#         b = self.lstm_len // 2
#
#         s = [b * 2, self.lstm_len, self.feature_len]
#         for H5_file in self.H5_file_list:
#             with h5py.File(H5_file, 'r') as h:
#                 full_edges_mask = np.ones(s)
#                 tmp1 = np.arange(1, self.lstm_len)
#                 front_edge = tmp1[:self.label_index_to_lstm_len]
#                 back_edge = tmp1[:self.lstm_len - self.label_index_to_lstm_len - 1]
#
#                 edge_ind = np.flip(front_edge)
#                 for i in front_edge:
#                     # print(i - 1, ':', edge_ind[i - 1])
#                     # print(full_edges_mask[i - 1, :edge_ind[i - 1], ...].shape)
#                     # print('\n')
#                     full_edges_mask[i - 1, :edge_ind[i - 1], ...] = np.zeros_like(
#                         full_edges_mask[i - 1, :edge_ind[i - 1], ...])
#
#                 edge_ind = np.flip(back_edge)
#                 for i in back_edge:
#                     # print(-i, -edge_ind[i - 1], ':')
#                     # print(full_edges_mask[-i, -edge_ind[i - 1]:, ...].shape)
#                     # print('\n')
#                     full_edges_mask[-i, -edge_ind[i - 1]:, ...] = np.zeros_like(
#                         full_edges_mask[-i, -edge_ind[i - 1]:, ...])
#
#                 all_edges = []
#                 for i1, i2 in utils.loop_segments(h['frame_nums']):  # 0, 1, 3998, 3999 ; 4000, 4001, 7998, 7999; ...
#                     edges = (np.asarray([[i1], [i2 - b]]) + np.arange(0, b).T).flatten()
#                     all_edges.append(edges)
#
#                 all_edges = np.asarray(all_edges)
#             self.all_edges_list.append(all_edges)
#             # pdb.set_trace()
#             full_edges_mask = full_edges_mask.astype(int)
#             self.full_edges_mask = full_edges_mask == 0
#
#
# h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing_features_data/feature_data/regular_80_border/DATA_FULL/3lag/holy_test_set_10_percent_3lag.h5'
# # h5 = '/Users/phil/Desktop/temp.h5'
# a = ImageBatchGenerator_feature_array(7, 4000, h5, label_key='labels', feature_len=2048,
#                                       label_index_to_lstm_len=3, edge_value=-1, remove_any_time_points_with_edges = False)
# x, y = a.__getitem__(0)
# # # utils.np_stats(x)
# # # s = x.shape;
# # # x2 = np.reshape(x, (s[0], s[1] * s[2]))
#
#
# plt.figure()
# plt.plot(y[2200:2300])
#
# plt.figure()
# plt.imshow(x[2200:2300, ::512].T)
#
#





import matplotlib.pyplot as plt
from tqdm.auto import tqdm

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py
from imgaug import augmenters as iaa  # optional program to further augment data

import lightgbm as lgb

import shap
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$ SET MATPLOTLIB DEFAULTS $$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
CB91_Blue = '#2CBDFE'
CB91_Green = '#47DBCD'
CB91_Pink = '#F3A0F2'
CB91_Purple = '#9D2EC5'
CB91_Violet = '#661D98'
CB91_Amber = '#F5B14C'

color_list = [CB91_Blue, CB91_Pink, CB91_Green, CB91_Amber,
              CB91_Purple, CB91_Violet]
plt.rcParams['axes.prop_cycle'] = plt.cycler(color=color_list)

import seaborn as sns

sns.set(rc={
 'axes.axisbelow': False,
 'axes.edgecolor': 'lightgrey',
 'axes.facecolor': 'None',
 'axes.grid': False,
 'axes.labelcolor': 'dimgrey',
 'axes.spines.right': False,
 'axes.spines.top': False,
 'figure.facecolor': 'white',
 'lines.solid_capstyle': 'round',
 'patch.edgecolor': 'w',
 'patch.force_edgecolor': True,
 'text.color': 'dimgrey',
 'xtick.bottom': False,
 'xtick.color': 'dimgrey',
 'xtick.direction': 'out',
 'xtick.top': False,
 'ytick.color': 'dimgrey',
 'ytick.direction': 'out',
 'ytick.left': False,
 'ytick.right': False})
sns.set_context("notebook", rc={"font.size":16,
                                "axes.titlesize":20,
                                "axes.labelsize":18})

## font

import matplotlib
matplotlib.rcParams['font.family'] = "sans-serif"
matplotlib.rcParams['font.sans-serif'] = "Arial"

plt.rcParams.update({'font.family':'Arial'})

# plt.rcParams.update({'font.family':'sans-serif', 'fontname':'Arial'})
from matplotlib.font_manager import findfont, FontProperties
font = findfont(FontProperties(family=['sans-serif']))
font
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$ SET MATPLOTLIB DEFAULTS $$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""


def load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0, label_key='labels'):
    if isinstance(h5_in, list):
        all_x = [];
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=split_int_n_chunks,
                                     split_chunk_ind=split_chunk_ind, label_key=label_key)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        all_y = np.hstack(all_y)
        return all_x, all_y
    all_x = None
    for k in feature_list:
        if all_x is None:
            all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype(
                'float32')
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype('float32')
            if len(x.shape) > 1:
                all_x = np.hstack((all_x, x))
            else:
                all_x = np.hstack((all_x, x[:, None]))

    all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[split_chunk_ind::split_int_n_chunks].astype(
        'float32')
    return all_x, all_y


def rm_nan_rows(a, a2=None):
    indx = ~np.isnan(a).any(axis=1)
    if a2 is None:
        return a[indx, :]
    else:
        return a[indx, :], a2[indx]


def get_feature_data_names(feature_list, n=2048):
    featuredata_names = [k.replace('FD__', '').replace('____', '') for k in feature_list]
    final_feature_names = []
    names = []
    nums = []
    for i, (k1, k2) in enumerate(zip(featuredata_names, feature_list)):
        if '_TOTAL_' in k2:
            names.append(k1)
            nums.append(np.arange(1))
            final_feature_names.append(k1)
        else:

            names.append(np.repeat(k1, n))
            nums.append(np.arange(n))
            final_feature_names.append([i1 + '_' + str(i2) for i1, i2 in zip(names[-1], nums[-1])])
    out = [np.hstack(k) for k in [final_feature_names, names, nums, featuredata_names]]
    out[0] = list(out[0])
    out[-1] = list(out[-1])
    return out

#
# h5_in = '/Users/phil/Desktop/LIGHT_GBM/reg_80_border_and_andrew_both_samson_both__test_num_0_percent_15.h5'
# mod_fn = '/Users/phil/Desktop/LIGHT_GBM/model_saves/model_num_0_of_10.pkl'
#
# with open(mod_fn, 'rb') as f:
#     model = pickle.load(f)
#
# explainer = shap.TreeExplainer(model)
#
# feature_list = natsorted(utils.lister_it(utils.print_h5_keys(h5_in, 1, 0), keep_strings='FD__'))
# final_feature_names, feature_names, feature_nums, feature_list_short = get_feature_data_names(feature_list)
# x_train, y_train = load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0)
#
#
# cut_index = 1030 + np.asarray([-50, 50])
#
# plt.plot(y_train[cut_index[0]:cut_index[1]])
#
# starti = 88
# sns.heatmap(x_train[:-41][cut_index[0]:cut_index[1], starti::2048//4].T)


'''
before these are all randomized i need to reference the final version that is very big 
i want fast touches with unique sidedness on each side 
access to images 
images that are good quality like jons or mine

'''
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$ SELECTING AN INFORMATIVE TOUCH TRACE $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

# h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'
# # here I am look for a good slice of touches in the orignal H5 file
# # training
# 383810 # aug - 3 border X 10
# 292217 # reg - 80 border (676027 - 383810)
# y = image_tools.get_h5_key_and_concatenate(h5_in)[383810:]
# smooth_by = 151
# ind = 0
#
# y2 = np.convolve(np.diff(y), np.ones(smooth_by)/smooth_by)
# bst_inds = np.flip(np.argsort(y2))
# add_to = smooth_by//2
# fig, ax = plt.subplots(5, 4)
# for ind, ax2 in enumerate(ax.flatten()):
#     ind+=100
#     ax2.plot(y[bst_inds[ind]-add_to:bst_inds[ind]+add_to])
#
# ind = 115
# add_to = 200
# plt.plot(y[bst_inds[ind]-add_to:bst_inds[ind]+add_to])
#
# plt.plot(y[271792:271792+200])
#
# plt.plot(y[271792:271792+200-50])
# 271792+383810
#
# with h5py.File(h5_in, 'r') as h:
#     test_image = h['images'][bst_inds[ind]+383810]
# plt.imshow(test_image)
# # CHECKING THE MAKE SURE THE IMAGES LOOK GOOD
# from whacc import analysis
# pp = analysis.pole_plot(h5_in)
# pp.current_frame = bst_inds[ind]+383810-20
# pp.plot_it()
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$ MAKE AN EXAMPLE H5 FOR SHOWING FEATURE ENGINEERING $$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
h5_in = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag/train_3lag.h5'

h5_new = '/Users/phil/Desktop/feature_example2.h5'
utils.print_h5_keys(h5_in)
with h5py.File(h5_in, 'r') as h:
    with h5py.File(h5_new, 'w') as h2:
        h2['images'] = h['images'][655102:656252]
        h2['labels'] = h['labels'][655102:656252]
        h2['frame_nums'] = [len(h2['labels'])]

with h5py.File(h5_new, 'r') as h:
    y3 = h['labels'][:]
    print(h['frame_nums'][:])


from whacc import model_maker
# convert to feature data
# h5_3lag = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag.h5'
h5_feature_data = h5_new.replace('.h5', '_feature_data.h5')
in_gen = image_tools.ImageBatchGenerator(500, h5_new, label_key='labels')
RESNET_MODEL = model_maker.load_final_model()
utils.convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)


# h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
# generate all the modified features (41*2048)+41 = 84,009
utils.standard_feature_generation(h5_feature_data)

#
#
# x = utils.get_whacc_path() + "/whacc_data/feature_data/"
# d = utils.load_obj(x + 'feature_data_dict.pkl')
# # x = d['features_used_of_10'][:]#>=4
# d['final_selected_features'] = utils.get_selected_features(greater_than_or_equal_to=4)
# x = d['final_selected_features'][:]
# d['final_selected_features_bool'] = [True if k in x else False for k in range(2048 * 41 + 41)]
#
# d = load_feature_data()
# def load_selected_features(h5_in, feature_index=None):
#     """
#
#     Parameters
#     ----------
#     h5_in : full directory of the full feature h5 file 84,009
#     feature_index : bool array of len 84,009
#
#     Returns
#     -------
#
#     """
#     d = utils.load_feature_data()
#     feature_list = d['feature_list_unaltered']
#     if feature_index is None:
#         feature_index= d['final_selected_features_bool']
#         # feature_index = utils.get_selected_features(greater_than_or_equal_to=4)
#     feature_index = utils.make_list(feature_index)
#
#     if isinstance(h5_in, list):
#         all_x = []
#         all_y = []
#         for k in h5_in:
#             tmp_x, tmp_y = load_selected_features(k, feature_list)
#             all_x.append(tmp_x)
#             all_y.append(tmp_y)
#             del tmp_x, tmp_y
#
#         all_x = np.vstack(all_x)
#         all_y = np.hstack(all_y)
#         return all_x, all_y
#
#     all_x = None
#     for k in tqdm(feature_list):
#         if all_x is None:
#             all_x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
#             inds = feature_index[:all_x.shape[1]]
#             del feature_index[:all_x.shape[1]]
#             all_x = all_x[:, inds]
#         else:
#             x = image_tools.get_h5_key_and_concatenate(h5_in, k).astype('float32')
#             if len(x.shape) > 1:
#                 inds = feature_index[:x.shape[1]]
#                 del feature_index[:x.shape[1]]
#                 x = x[:, inds]
#                 all_x = np.hstack((all_x, x))
#             else:
#                 inds = feature_index.pop(0)  # single true or false to include the "TOTAL" variables
#                 if inds:
#                     all_x = np.hstack((all_x, x[:, None]))
#     return all_x


x = utils.get_whacc_path() + "/whacc_data/feature_data/"
d = utils.load_obj(x + 'feature_data_dict.pkl')
# x = d['features_used_of_10'][:]#>=4
d['final_selected_features'] = np.where(d['features_used_of_10'][:]>=4)[0]
x = d['final_selected_features'][:]
d['final_selected_features_bool'] = [True if k in x else False for k in range(2048 * 41 + 41)]

all_x = utils.load_selected_features(h5_feature_data)

with h5py.File(h5_feature_data, 'r+') as h:
    h['final_3095_features'] = all_x



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$ NOW PLOT IT TO SHOW THE READER  what the features look like$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""


"""$$$$$$$$$$$$$$$$$$$$$$$$$ grab the correct index $$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
import scipy.cluster.hierarchy as sch
def cluster_corr(corr_array, max_div_by = 2):
    pairwise_distances = sch.distance.pdist(corr_array)
    linkage = sch.linkage(pairwise_distances, method='complete')
    cluster_distance_threshold = pairwise_distances.max()/max_div_by
    idx_to_cluster_array = sch.fcluster(linkage, cluster_distance_threshold, criterion='distance')
    idx = np.argsort(idx_to_cluster_array)
    if isinstance(corr_array, pd.DataFrame):
        return corr_array.iloc[idx, :].T.iloc[idx, :]
    return corr_array[idx, :][:, idx], idx

h5_feature_data = '/Users/phil/Desktop/feature_example2_feature_data.h5'
utils.print_h5_keys(h5_feature_data)

with h5py.File(h5_feature_data, 'r') as h:
    d = dict()
    for k in h.keys():
        d[k] = copy.deepcopy(h[k][500:500+150])

# with h5py.File(h5_feature_data, 'r') as h:
#     d = dict()
#     for k in h.keys():
#         d[k] = copy.deepcopy(h[k][:])

def foo_neuron_heatmap(FD, inds, labels, only_return_data = False):
    # sns.heatmap(cc)
    data = FD[:, inds].T
    data = data-np.min(data)
    data = data/np.max(data)
    if only_return_data:
        return data
    fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
    plt.sca(ax[0])
    ax_heat = sns.heatmap(data, cbar=False)
    x = np.linspace(0, 2047, 5).astype(int)
    ax_heat.set_yticks(x)
    ax_heat.set_yticklabels(x+1)
    plt.ylabel('2048 CNN output neurons')
    plt.sca(ax[1])
    plt.plot(labels, 'k-', label='touch trace')
    ax[1].set_yticklabels([])
    x = np.arange(0, 151, 50).astype(int)
    ax[1].set_xticks(x)
    ax[1].set_xticklabels(x)
    plt.xlabel('time (ms)')
    plt.tight_layout()
    plt.legend(bbox_to_anchor=(.8, 1.85), loc=2, borderaxespad=0, fontsize=10)
    plt.sca(ax[0])
    return data
"""$$$$$$$$$$$$$$$$$$$$$$$$$ to plot each $$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

labels = d['labels']
FD = d['FD__original']
FD[0, :] = FD[0, :]+0.00000001 # so that CC works, prevents values form one neuron all being the same
# 1 method use clustering
cc = np.corrcoef(FD, rowvar=False)
cc, inds = cluster_corr(cc, 2)


data = foo_neuron_heatmap(FD, inds, labels)

from matplotlib.patches import Rectangle
plt.gca().add_patch(Rectangle((40 - .5, 755 - .5), 30, 50, facecolor='none'))


ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')

plt.ylim([780+25, 780-25])
plt.xlim([40, 70])
plt.figure()

sns.heatmap(data[780-25:780+25, 40:70])

plt.show(block=False)
plt.close('all')

plt.legend(bbox_to_anchor=(.8, 1.85), loc=2, borderaxespad=0, fontsize=10)


fd = d['FD__FD__original_diff_periods_-50____']
foo_neuron_heatmap(fd, inds, labels)

fd = d['FD__FD__original_diff_periods_-2____']
foo_neuron_heatmap(fd, inds, labels)

from natsort import natsorted
keys = utils.print_h5_keys(h5_feature_data, 1, 0)
keys = natsorted(utils.lister_it(keys, 'FD__', 'TOTAL'))
cut_list = []
for k in keys:
    fd = d[k]
    data = foo_neuron_heatmap(fd, inds, labels, only_return_data = True)
    cut_list.append(data[780-25:780+25, 40:70])

x = np.hstack(cut_list[:16])
ax = sns.heatmap(x)
ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')


x = np.hstack(cut_list[16:23])
ax = sns.heatmap(x)
ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')

x = np.hstack(cut_list[23:30])
ax = sns.heatmap(x)
ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')

x = np.hstack(cut_list[30:40])
ax = sns.heatmap(x)
ax.vlines(np.arange(0, x.shape[1], 30), *ax.get_xlim(), 'w')

keys[:16] # diff
keys[16:23] # rolling mean
keys[23:30] # rolling std
keys[30:40] # shift

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$ PLOT TOTAL STD FEATURES   $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
keys = utils.print_h5_keys(h5_feature_data, 1, 0)
keys = natsorted(utils.lister_it(keys, 'TOTAL'))
total_type_features = []
for k in keys:
    fd = d[k]
    total_type_features.append(fd)

total_type_features = np.vstack(total_type_features)

key_labels = []
for k in keys:
    k2 = ' '.join(k.split('FD__original_')[-1].split('_')[:4])
    key_labels.append(k2)
    print(k2)
key_labels[-1] = 'Original'

fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
plt.sca(ax[0])

ax_heat = sns.heatmap(total_type_features, cbar=False, yticklabels=key_labels)
ax_heat.set_yticklabels(key_labels, fontsize= 7)

# x = np.linspace(0, 40, 5).astype(int)
# ax_heat.set_yticks(x)
# ax_heat.set_yticklabels(x)
plt.ylabel('TOTAL SD features (2048 reduced to 1)')
plt.sca(ax[1])
plt.plot(labels, 'k-', label='touch trace')
ax[1].set_yticklabels([])
x = np.arange(0, 151, 50).astype(int)
ax[1].set_xticks(x)
ax[1].set_xticklabels(x)
plt.xlabel('time (ms)')
plt.tight_layout()
plt.legend(bbox_to_anchor=(.8, 1.65), loc=2, borderaxespad=0, fontsize=10)
plt.sca(ax[0])

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$ PLOT 3095 features   $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
def foo_norm(x):################################################
    x = x-np.min(x)
    x = x/np.max(x)
    return x
# def foo_norm(x): ################################################
#     return x
FD = d['final_3095_features']

FD2 = copy.deepcopy(FD)
for i, k in enumerate(FD.T):
    FD2[:, i] = foo_norm(k)
FD = FD2
FD[0, :] = FD[0, :]+0.00000001
cc = np.corrcoef(FD, rowvar=False)
cc, inds = cluster_corr(cc, 1.8)



# inds = np.arange(FD.shape[1]) ################################################



data = FD[:, inds].T
sns.heatmap(cc)

fig, ax = plt.subplots(2, 1, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
plt.sca(ax[0])

ax_heat = sns.heatmap(data, cbar=False)
# ax_heat.set_yticklabels(key_labels, fontsize= 7)

# x = np.linspace(0, 40, 5).astype(int)
# ax_heat.set_yticks(x)
# ax_heat.set_yticklabels(x)
plt.ylabel('______')
plt.sca(ax[1])
plt.plot(d['labels'], 'k-', label='touch trace')


ax[1].set_yticklabels([])
x = np.arange(0, 151, 50).astype(int)
ax[1].set_xticks(x)
ax[1].set_xticklabels(x)
plt.xlabel('time (ms)')
plt.tight_layout()
plt.legend(bbox_to_anchor=(.8, 1.65), loc=2, borderaxespad=0, fontsize=10)
plt.sca(ax[0])



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$ test loaded model on small set   $$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""



model_save_test = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/top_features_greter_than_3_out_of_10_3095_features/model_num_0_of_10.pkl'
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models/top_features_greter_than_3_out_of_10_3095_features/'
for i, model_save_test in enumerate(utils.get_files(bd, '*of_10.pkl')):
    mod = utils.load_obj(model_save_test)
    # d['final_3095_features']
    pred_y = mod.predict(d['final_3095_features'])
    plt.plot(pred_y+i+1, '-', label='pred trace')



x = utils.get_whacc_path() + "/whacc_data/feature_data/"
d = utils.load_obj(x + 'feature_data_dict.pkl')

x = d['feature_list_unaltered']

from natsort import ns
from natsort import natsorted, humansorted

natsorted(x, alg=ns.INT | ns.LOCALE | ns.PATH)
natsorted(x)
humansorted(x)

natsorted(x, alg=ns.REAL)






def foo(x, remove_inds, axis = 0):
    x = np.swapaxes(x, 0, axis)
    save_x = []
    for k in remove_inds:
        save_x.append(copy.deepcopy(x[k]))
        x = np.delete(x, k)
    return x, save_x

"""
so below works but want to se if it works without giving memory errors 
"""
import os

# Getting all memory using os.popen()
import copy
import pandas as pd
import numpy as np
from psutil import Process, virtual_memory
def mem():
    print('RAM memory % used:', virtual_memory()[2])

del x
mem()
a, b = 6000, 800000
x = pd.DataFrame(np.arange(a*b).reshape(a, b))
mem()
tmp1 = pd.DataFrame([])
pop_inds = [1, 3]
for i, k in enumerate(pop_inds):
    tmp1.insert(i, i, x.pop(k))
mem()
print(tmp1)
print(x)
for i, k in enumerate(pop_inds):
    x.insert(k, k, tmp1.pop(i))
mem()
print(x)



x2, save_x = foo(x, [2, 3], axis = 0)
x, x2, save_x



def foo(x, axis = 0):
    assert axis is in [0, 1], 'I can only deal with 2 dim data'
    out = copy.deepcopy(x[])
    return x, out

all_mean_shap = []
x = np.arange(6*4).reshape(6, 4)
x
rows = [10, 20, 25]
col = [2, 4]
for i in range(5):
    for c in
    tmp_save_x = copy.deepcopy(all_val_sets[k1:k2, :])
    print(tmp_save_x.shape)
    all_val_sets = np.delete(all_val_sets, np.arange(k1, k2), axis = 0) # delete chunk
    print(all_val_sets.shape)
    # PREDICT
    shap_vals = lgbm.predict(all_val_sets, pred_contrib = True)


    np.abs(shap_vals, out = shap_vals) # this prevent mem error, shap_vals is modified here
    mean_shape = np.mean(shap_vals, axis = 0)
    all_mean_shap.append(mean_shape)

    del shap_vals
    gc.collect()

    all_val_sets = np.insert(all_val_sets, k1, tmp_save_x, axis = 0) # insert chunk back in


